@@ -15,7 +15,8 @@
  * 
  *     You should have received a copy of the GNU General Public
  *     License along with this program; if not, write to the Free
- *     Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Software Foundation, Inc., 59 Temple Place - Suite 330,
+ *     Boston, MA  02111, USA.
  */ 
 
 /* simple, editor-based message editing */
@@ -126,7 +127,7 @@ static int edit_one_message (CONTEXT *ctx, HEADER *cur)
 
   of = cf = 0;
   
-  if (fgets (buff, sizeof (buff), fp) && is_from (buff, NULL, 0))
+  if (fgets (buff, sizeof (buff), fp) && is_from (buff, NULL, 0, NULL))
   {
     if (tmpctx.magic == M_MBOX || tmpctx.magic == M_MMDF ||
 	tmpctx.magic == M_KENDRA)
