@@ -153,91 +153,120 @@ void mutt_decode_xbit (STATE *s, BODY *b, int istext, iconv_t cd)
     mutt_copy_bytes (s->fpin, s->fpout, len);
 }
 
-static int handler_state_fgetc(STATE *s)
+static int qp_decode_triple (char *s, char *d)
 {
-  int ch;
+  /* soft line break */
+  if (*s == '=' && !(*(s+1)))
+    return 1;
   
-  if((ch = fgetc(s->fpin)) == EOF)
+  /* quoted-printable triple */
+  if (*s == '=' && isxdigit (*(s+1)) && isxdigit (*(s+2)))
   {
-    dprint(1, (debugfile, "handler_state_fgetc: unexpected EOF.\n"));
-    state_puts (_("[-- Error: unexpected end of file! --]\n"), s);
+    *d = (hexval (*(s+1)) << 4) | hexval (*(s+2));
+    return 0;
   }
-  return ch;
+  
+  /* something else */
+  return -1;
 }
 
+static void qp_decode_line (char *dest, char *src, size_t *l)
+{
+  char *d, *s;
+  char c;
+
+  int kind;
+  int soft = 0;
+
+  /* chop trailing whitespace */
+
+  if (*src)
+  {
+    s = src + strlen(src) - 1;
+    while (s >= src && ISSPACE(*s))
+      s--;
+    *(++s) = '\0';
+  }
+
+  /* decode the line */
+  
+  for (d = dest, s = src; *s;)
+  {
+    switch ((kind = qp_decode_triple (s, &c)))
+    {
+      case  0: *d++ = c; s += 3; break;	/* qp triple */
+      case -1: *d++ = *s++;      break; /* single character */
+      case  1: soft = 1; s++;	 break; /* soft line break */
+    }
+  }
+
+  if (!soft)
+    *d++ = '\n';
+  
+  *d = '\0';
+  *l = d - dest;
+}
+
+/* 
+ * Decode an attachment encoded with quoted-printable.
+ * 
+ * Why doesn't this overflow any buffers?  First, it's guaranteed
+ * that the length of a line grows when you _en_-code it to
+ * quoted-printable.  That means that we always can store the
+ * result in a buffer of at most the _same_ size.
+ * 
+ * Now, we don't special-case if the line we read with fgets()
+ * isn't terminated.  We don't care about this, since STRING > 78,
+ * so corrupted input will just be corrupted a bit more.  That
+ * implies that STRING+1 bytes are always sufficient to store the
+ * result of qp_decode_line.
+ * 
+ * Finally, at soft line breaks, some part of a multibyte character
+ * may have been left over by convert_to_state().  This shouldn't
+ * be more than 6 characters, so STRING + 7 should be sufficient
+ * memory to store the decoded data.
+ * 
+ * Just to make sure that I didn't make some off-by-one error
+ * above, we just use STRING*2 for the target buffer's size.
+ * 
+ */
+
 void mutt_decode_quoted (STATE *s, BODY *b, int istext, iconv_t cd)
 {
   long len = b->length;
-  int ch;
-  char bufi[BUFI_SIZE];
+  char line[STRING];
+  char decline[2*STRING];
   size_t l = 0;
+  size_t l2;
 
   state_set_prefix(s);
 
   while (len > 0)
   {
-    if ((ch = handler_state_fgetc(s)) == EOF)
+    if (fgets (line, sizeof (line), s->fpin) == NULL)
       break;
 
-    len--;
-    
-    if (ch == '=')
-    {
-      int ch1, ch2;
-      
-      if(!len || (ch1 = handler_state_fgetc(s)) == EOF)
-	break;
+    len -= (l2 = strlen (line));
 
-      len--;
-      
-      if (ch1 == '\n' || ch1 == '\r' || ch1 == ' ' || ch1 == '\t')
-      {
-	/* Skip whitespace at the end of the line since MIME does not
-	 * allow for it
-	 */
-	if(ch1 != '\n')
-	{
-	  while(len && (ch1 = handler_state_fgetc(s)) != EOF)
-	  {
-	    len--;
-	    if(ch1 == '\n')
-	      break;
-	  }
-	}
-
-	if(ch1 == EOF)
-	  break;
-
-	ch = EOF;
-
-      }
-      else
-      {
-	if(!len || (ch2 = handler_state_fgetc(s)) == EOF)
-	  break;
+    /* 
+     * Skip over pending input data until end of line - at this
+     * point, the only thing we may see is trailing whitespace,
+     * i.e. garbage.
+     */
 
+    if (l2 && line[l2 - 1] != '\n')
+      while (len > 0 && fgetc (s->fpin) != '\n')
 	len--;
-	
-        ch = hexval (ch1) << 4;
-        ch |= hexval (ch2);
-      }
-    } /* ch == '=' */
-    else if (istext && ch == '\r')
-    {
-      continue;
-    }
 
-    if(ch != EOF)
-    {
-      bufi[l++] = ch;
-      if (l == sizeof (bufi))
-	convert_to_state (cd, bufi, &l, s);
-    }
+    /* 
+     * decode and do character set conversion
+     */
+    
+    qp_decode_line (decline + l, line, &l);
+    convert_to_state (cd, decline, &l, s);
   }
 
-  convert_to_state (cd, bufi, &l, s);
   convert_to_state (cd, 0, 0, s);
-  
   state_reset_prefix(s);
 }
 
@@ -337,7 +366,7 @@ void mutt_decode_uuencoded (STATE *s, BODY *b, int istext, iconv_t cd)
   char *pt;
   long len = b->length;
   char bufi[BUFI_SIZE];
-  size_t k;
+  size_t k = 0;
 
   if(istext)
     state_set_prefix(s);
