@@ -31,7 +31,7 @@
 #endif
 
 
-#ifdef USE_SSL
+#if defined(USE_SSL) || defined(USE_NSS)
 #include "mutt_ssl.h"
 #endif
 
@@ -1786,11 +1786,6 @@ void mutt_init (int skip_sys_rc, LIST *commands)
   }
 
   Tempdir = safe_strdup ((p = getenv ("TMPDIR")) ? p : "/tmp");
-
-#ifdef USE_POP
-  PopUser = safe_strdup (Username);
-#endif
-
   Editor = safe_strdup ((p = getenv ("EDITOR")) ? p : "vi");
   Visual = safe_strdup ((p = getenv ("VISUAL")) ? p : Editor);
 
@@ -1809,12 +1804,7 @@ void mutt_init (int skip_sys_rc, LIST *commands)
     FREE (&token.data);
   }
 
-#ifdef HAVE_LANGINFO_CODESET
   mutt_set_langinfo_charset ();
-#else
-  Charset = safe_strdup ("iso-8859-1");
-#endif
-
   mutt_set_charset (Charset);
   
   
