@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #ifndef _MAILBOX_H
@@ -51,8 +51,8 @@ MESSAGE *mx_open_new_message (CONTEXT *, HEADER *, int);
 
 void mx_fastclose_mailbox (CONTEXT *);
 
-int mx_close_mailbox (CONTEXT *);
-int mx_sync_mailbox (CONTEXT *);
+int mx_close_mailbox (CONTEXT *, int *);
+int mx_sync_mailbox (CONTEXT *, int *);
 int mx_commit_message (MESSAGE *, CONTEXT *);
 int mx_close_message (MESSAGE **);
 int mx_get_magic (const char *);
@@ -61,6 +61,9 @@ int mx_check_mailbox (CONTEXT *, int *, int);
 #ifdef USE_IMAP
 int mx_is_imap (const char *);
 #endif
+#ifdef USE_POP
+int mx_is_pop (const char *);
+#endif
 
 
 #endif
