@@ -41,7 +41,11 @@
 #define ENCWORD_LEN_MAX 75
 #define ENCWORD_LEN_MIN 9 /* strlen ("=?.?.?.?=") */
 
-#define HSPACE(x) ((x) == ' ' || (x) == '\t')
+#define HSPACE(x) ((x) == '\0' || (x) == ' ' || (x) == '\t')
+
+#define CONTINUATION_BYTE(c) (((c) & 0xc0) == 0x80)
+
+extern char RFC822Specials[];
 
 typedef size_t (*encoder_t) (char *, const char *, size_t,
 			     const char *);
@@ -326,7 +330,7 @@ static size_t choose_block (char *d, size_t dlen, int col,
     if (!nn && col + *wlen <= ENCWORD_LEN_MAX + 1)
       break;
     nn = (nn ? nn : n) - 1;
-    while ((d[nn] & 0xc0) == 0x80)
+    while (CONTINUATION_BYTE(d[nn]))
       --nn;
     assert (d + nn >= d);
     if (!nn)
@@ -348,12 +352,13 @@ static size_t choose_block (char *d, size_t dlen, int col,
  */
 static int rfc2047_encode (const char *d, size_t dlen, int col,
 			   const char *fromcode, const char *charsets,
-			   char **e, size_t *elen)
+			   char **e, size_t *elen, char *specials)
 {
   int ret = 0;
   char *buf;
   size_t bufpos, buflen;
   char *u, *t0, *t1, *t;
+  char *s0, *s1;
   size_t ulen, r, n, wlen;
   encoder_t encoder;
   char *tocode1 = 0;
@@ -363,23 +368,41 @@ static int rfc2047_encode (const char *d, size_t dlen, int col,
   /* Try to convert to UTF-8. */
   if (convert_string (d, dlen, fromcode, icode, &u, &ulen))
   {
-    ret = 1, icode = 0;
-    u = safe_malloc (ulen = dlen), memcpy (u, d, dlen);
+    ret = 1; 
+    icode = 0;
+    u = safe_malloc ((ulen = dlen) + 1);
+    memcpy (u, d, dlen);
+    u[ulen] = 0;
   }
 
   /* Find earliest and latest things we must encode. */
-  t0 = t1 = 0;
-  for (t = u; t < u + ulen - 1; t++)
-    if ((*t & 0x80) ||
+  s0 = s1 = t0 = t1 = 0;
+  for (t = u; t < u + ulen; t++)
+  {
+    if ((*t & 0x80) || 
 	(*t == '=' && t[1] == '?' && (t == u || HSPACE(*(t-1)))))
-      t0 = t0 ? t0 : t, t1 = t;
-  if (t == u + ulen - 1 && (*t & 0x80))
-    t0 = t0 ? t0 : t, t1 = t;
+    {
+      if (!t0) t0 = t;
+      t1 = t;
+    }
+    else if (specials && strchr (specials, *t))
+    {
+      if (!s0) s0 = t;
+      s1 = t;
+    }
+  }
+
+  /* If we have something to encode, include RFC822 specials */
+  if (t0 && s0 && s0 < t0)
+    t0 = s0;
+  if (t1 && s1 && s1 > t1)
+    t1 = s1;
 
   if (!t0)
   {
     /* No encoding is required. */
-    *e = u, *elen = ulen;
+    *e = u;
+    *elen = ulen;
     return ret;
   }
 
@@ -393,17 +416,22 @@ static int rfc2047_encode (const char *d, size_t dlen, int col,
       ret = 2, icode = 0;
   }
 
+  /* Hack to avoid labelling 8-bit data as us-ascii. */
+  if (!icode && mutt_is_us_ascii (tocode))
+    tocode = "unknown-8bit";
+  
   /* Adjust t0 for maximum length of line. */
   t = u + (ENCWORD_LEN_MAX + 1) - col - ENCWORD_LEN_MIN;
-  t = t > u ? t : u;
-  t0 = t < t0 ? t : t0;
+  if (t < u)  t = u;
+  if (t < t0) t0 = t;
+  
 
   /* Adjust t0 until we can encode a character after a space. */
   for (; t0 > u; t0--)
   {
     if (!HSPACE(*(t0-1)))
       continue;
-    for (t = t0 + 1; t < u + ulen && (*t & 0xc0) == 0x80; t++)
+    for (t = t0 + 1; t < u + ulen && CONTINUATION_BYTE(*t); t++)
       ;
     if (!try_block (t0, t - t0, icode, tocode, &encoder, &wlen) &&
 	col + (t0 - u) + wlen <= ENCWORD_LEN_MAX + 1)
@@ -415,7 +443,7 @@ static int rfc2047_encode (const char *d, size_t dlen, int col,
   {
     if (!HSPACE(*t1))
       continue;
-    for (t = t1 - 1; (*t & 0xc0) == 0x80; t--)
+    for (t = t1 - 1; CONTINUATION_BYTE(*t); t--)
       ;
     if (!try_block (t, t1 - t, icode, tocode, &encoder, &wlen) &&
 	1 + wlen + (u + ulen - t1) <= ENCWORD_LEN_MAX + 1)
@@ -442,7 +470,7 @@ static int rfc2047_encode (const char *d, size_t dlen, int col,
       /* See if we can fit the us-ascii suffix, too. */
       if (col + wlen + (u + ulen - t1) <= ENCWORD_LEN_MAX + 1)
 	break;
-      for (n = t1 - t - 1; (t[n] & 0xc0) == 0x80; n--)
+      for (n = t1 - t - 1; CONTINUATION_BYTE(t[n]); n--)
 	;
       assert (t + n >= t);
       if (!n)
@@ -494,7 +522,7 @@ static int rfc2047_encode (const char *d, size_t dlen, int col,
   return ret;
 }
 
-void rfc2047_encode_string (char **pd)
+void _rfc2047_encode_string (char **pd, int encode_specials, int col)
 {
   char *e;
   size_t elen;
@@ -507,10 +535,9 @@ void rfc2047_encode_string (char **pd)
   if (!charsets || !*charsets)
     charsets = "UTF-8";
 
-  /* Pretend that we are starting in column 32, thus allowing for a
-     field-name with up to 30 characters. */
-  rfc2047_encode (*pd, strlen (*pd), 32,
-		  Charset, charsets, &e, &elen);
+  rfc2047_encode (*pd, strlen (*pd), col,
+		  Charset, charsets, &e, &elen,
+		  encode_specials ? RFC822Specials : NULL);
 
   safe_realloc ((void **) &e, elen + 1);
   e[elen] = '\0';
@@ -518,17 +545,18 @@ void rfc2047_encode_string (char **pd)
   *pd = e;
 }
 
-void rfc2047_encode_adrlist (ADDRESS *addr)
+void rfc2047_encode_adrlist (ADDRESS *addr, const char *tag)
 {
   ADDRESS *ptr = addr;
-
+  int col = tag ? strlen (tag) + 2 : 32;
+  
   while (ptr)
   {
     if (ptr->personal)
-      rfc2047_encode_string (&ptr->personal);
+      _rfc2047_encode_string (&ptr->personal, 1, col);
 #ifdef EXACT_ADDRESS
     if (ptr->val)
-      rfc2047_encode_string (&ptr->val);
+      _rfc2047_encode_string (&ptr->val, 1, col);
 #endif
     ptr = ptr->next;
   }
